<?php
namespace Smartlink\Checkout\Observer;

use Magento\Framework\Event\ObserverInterface;
use Smartlink\Checkout\Model\Api\Api;
use Magento\Framework\HTTP\ZendClientFactory;
use Zend\Http\Request;
class SmartlinkRedirect implements ObserverInterface
{
  protected $_redirect;
  protected $_response;
  protected $apiCall;
  protected $_scopeConfig;
  protected $request;
  protected $_messageManager;
  protected $_url;
  protected $_debug_logger;

  public function __construct(
    \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig, 
    \Magento\Framework\UrlInterface $url,
    \Magento\Framework\App\Response\RedirectInterface $redirect,
    \Magento\Framework\App\ResponseInterface $response,
    \Smartlink\Checkout\Model\Api\Api $apiCall,
    \Magento\Framework\App\RequestInterface $request,
    \Magento\Framework\Message\ManagerInterface $messageManager,
    \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
    \Magento\Sales\Model\Service\InvoiceService $invoiceService,
    \Psr\Log\LoggerInterface $debug_logger
 ) {

   $this->_redirect = $redirect;
   $this->_response = $response;
   $this->_url = $url;
   $this->apiCall = $apiCall;
   $this->_scopeConfig = $scopeConfig;
   $this->request = $request;
   $this->_messageManager = $messageManager;
   $this->productRepository = $productRepository;
   $this->_invoiceService = $invoiceService;
}

public function execute(\Magento\Framework\Event\Observer $observer)
{       
 

   $order_ids = $observer->getEvent()->getOrderIds();
   $order_id = $order_ids[0];
   $order = $this->getOrder($order_id);
   if ($order->getPayment()->getMethodInstance()->getCode() == 'smartlink_checkout') {
      
      $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
      $connection = $resource->getConnection();
      $sql = "SELECT * FROM smartlink_payments WHERE order_id= '".$order_id."'";
      $result = $connection->fetchAll($sql);
      if(empty($result)){

         $merchantId = $this->getStoreConfig('payment/smartlink_checkout/marchant_id');
         $apiKey = $this->getStoreConfig('payment/smartlink_checkout/api_key');
         $webId = $this->getStoreConfig('payment/smartlink_checkout/web_id');
         if($merchantId != '' && $apiKey != ''){
            $items = $order->getAllItems();
            $currencyCode = $order->getOrderCurrencyCode();
            $grandTotal = $order->getGrandTotal();
            // $validationData = [
               //    "api_key" => $apiKey,
               //    "merchant_id" => $merchantId,
               //    "fiat_currency" => $currencyCode,
               //    "fiat_amount" => $grandTotal,
               // ];
              
            // $checkValidation = $this->validate_data($validationData);
            $description = [];
            foreach ($items as $item) {
               array_push($description,$item->getName());
            }
            $description = implode(',',$description);
            $sentData = [
               "web_id"=>$webId,
               "order_id"=>$order_id,
               "fiat_currency" => $currencyCode,
               "fiat_amount" => $grandTotal,
               "description" => $description
            ];
            
            $storeData = $this->store_data($sentData);
            if($storeData['flag']==1){
               $db_payment = [
                  'order_id' => $order_id,
                  'fiat_amount' => $grandTotal,
                  'fiat_currency' => $currencyCode,
                  'status' => 0,
               ];
               $this->dataInsertDB('smartlink_payments',$db_payment);
               $redirectUrl = $this->getBaseUrl().'smartlink/invoice/Index/order/'.$order_id;
               
               return $this->_redirect->redirect($this->_response, $redirectUrl);
            }
            
         }else{
            $msg = "Opps! Somethig went wrong!";
         }

         //delete order
         $registry = $objectManager->get('Magento\Framework\Registry');
         $registry->register('isSecureArea','true');
         $order->delete();
         $registry->unregister('isSecureArea');
         
         if(!isset($msg)){
            $msg = 'Opps! Somethig went wrong!';
         }
         $this->_messageManager->addError($msg);
         $cartUrl = $this->_url->getUrl('checkout/cart/index');
         $this->_redirect->redirect($this->_response, $cartUrl);
      } else {
         $status = $result[0]['status'];
         if($status == 1 || $status == 3){
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $orders = $objectManager->create('\Magento\Sales\Model\Order')->load($order_id);
            $orderState = $this->getStoreConfig('payment/smartlink_checkout/order_status');
            $orders->setStatus($orderState);
            $orders->save();
            $this->downloadableOrder($order_id);
         }
      }
   }
    } //end execute function

    public function getOrder($_order_id){

      $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      $order = $objectManager->create('Magento\Sales\Api\Data\OrderInterface')->load($_order_id);
      return $order;

   }

   public function getBaseUrl(){

      $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
      return $storeManager->getStore()->getBaseUrl();

   }

   public function getStoreConfig($_env){

      $_val = $this->_scopeConfig->getValue($_env, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
      return $_val;
   }

   public function dataInsertDB($_table, $_data){

      $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
      $connection = $resource->getConnection();
      $connection->insertForce($_table,$_data);
   }
   
   public function validate_data($param){

      $api_base_url = $this->apiCall->getApiUrl();
      $data = $param;
      $url =  $api_base_url."/validate-data";
      $res = $this->apiCall->apiCaller($url, Request::METHOD_POST,$data);
      return $res;
   }
   public function store_data($param){

      $api_base_url = $this->apiCall->getApiUrl();
      $data = $param;
      $url =  $api_base_url."/store-order-to-cache";
      $res = $this->apiCall->apiCaller($url, Request::METHOD_POST,$data);
      return $res;
   }

   public function downloadableOrder($orderId)
   {
      $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      $order = $objectManager->create('Magento\Sales\Api\Data\OrderInterface')->load($orderId);
      $orderItems = $order->getAllItems();
      $isDownloadableType = null;
      foreach ($orderItems as $item) {
         $product = $this->productRepository->get($item->getSku());
         if ($product->getTypeId() === \Magento\Downloadable\Model\Product\Type::TYPE_DOWNLOADABLE) {
            $isDownloadableType = true; 
            break;
         }
      }
      if($isDownloadableType){
         $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
         $connection = $resource->getConnection();
         $status = \Magento\Downloadable\Model\Link\Purchased\Item::LINK_STATUS_AVAILABLE;
         $sql = "UPDATE downloadable_link_purchased_item SET status='$status' WHERE purchased_id IN (SELECT purchased_id FROM downloadable_link_purchased WHERE order_id=$orderId)";
         $result = $connection->query($sql);
      }
   }
   
}
