<?php

namespace Smartlink\Checkout\Ui\Component\Listing\Column;

use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Ui\Component\Listing\Columns\Column;
use Magento\Framework\UrlInterface;

class WalletsActions extends Column
{
    /** Url path */
    const ROW_EDIT_URL = 'smartlink/wallets/edit';
    const ROW_DELETE_URL = 'smartlink/wallets/delete';
    /** @var UrlInterface */
    protected $_urlBuilder;
    
    /**
     * @var string
     */
    private $_editUrl;
    
    /**
     * @param ContextInterface   $context
     * @param UiComponentFactory $uiComponentFactory
     * @param UrlInterface       $urlBuilder
     * @param array              $components
     * @param array              $data
     * @param string             $editUrl
     */
    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        UrlInterface $urlBuilder,
        array $components = [],
        array $data = [],
        $editUrl = self::ROW_EDIT_URL
    ) 
    {
        $this->_urlBuilder = $urlBuilder;
        $this->_editUrl = $editUrl;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }
    
    /**
     * Prepare Data Source.
     *
     * @param array $dataSource
     *
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        if (isset($dataSource['data']['items'])) {
            foreach ($dataSource['data']['items'] as &$item) {
                $name = $this->getData('name');
                if (isset($item['id'])) {
                    $item[$name]['edit'] = [
                        'href' => $this->_urlBuilder->getUrl(
                            $this->_editUrl, 
                            ['id' => $item['id']]
                        ),
                        'label' => __('Edit'),
                    ];

                    $title = $item['name'];
                    $item[$name]['delete'] = [
                        'href' => $this->_urlBuilder->getUrl(
                            self::ROW_DELETE_URL, 
                            ['id' => $item['id']]
                        ),
                        'label' => __('Delete'),
                        'confirm'=>[
                            'title'=>__('Delete %1', $title),
                            'message'=>__('Are you sure you want to delete a %1 wallet?', $title)
                        ],
                    ];
                }
            }
        }
        
        return $dataSource;
    }
}